#allsymblist_v1-0-1.pl
#command line:
#script.pl input.txt output.txt

use strict;
use warnings;
use open ':utf8';
use Encode;
use charnames ':full';
use POSIX qw(ceil floor);

my %hash = ();
my $count = 0;

sub trim
{
	my $s = shift;
	$s =~ s/^\s+//;
	$s =~ s/\s+$//;
	return $s
}

sub codepoint_hex {
	if (my $char = shift) {
		return sprintf '%2.2x', unpack('U0U*', $char);
	}
}

sub pad {
	my $str = shift;
	while (length $str < 4){
		$str = "0$str";
	}
	return "0x$str";
}

sub progressinfile {
	my $state = shift;
	my $max = shift;
	my $message = shift;

	my $percent = ($state/$max)*100;
	$percent = floor($percent);

	for(my $k=0;$k<$percent;$k=$k+5)
	{
		print "|";						
	}

	print "$percent% $message";
	print "\r";

}



if (@ARGV != 2)
{
	print "  ________________________________\n\n";
	print "  Invalid command line.\n";
	print "  ________________________________\n\n";
	exit;
}


open(A, "$ARGV[0]") || die "could not open file: $!";
open(B, ">$ARGV[1]") || die "could not open file: $!";

my $filesize = -s A;

while (<A>) {

	s/[\x0a\x0d]//g;

	if ($_ =~ /^\x{feff}(.*)$/)
	{
		$_ = $1;
	}


	my @arr = split(//, $_);

	$count++;
	if (($count % 2) == 0)
	{
		my $offset = tell(A);
		progressinfile($offset, $filesize, "");
	}


	foreach(@arr){

		if (exists $hash{"$_"})
		{
			my $c = $hash{"$_"};
			$c++;
			$hash{"$_"} = $c;
		}
		else
		{
			$hash{"$_"} = 1;
		}
	}

}

my $offset = tell(A);
progressinfile($offset, $filesize, "");
print "\nOK\n";

close A;

my @k = keys %hash;

my @sorted = sort{ $hash{$b} <=> $hash{$a} } @k;

foreach my $r (@sorted)
{
	my $cp = pad(codepoint_hex($r));
	my $oc = encode("UTF-8", $r);
	my $h = unpack("H*", $oc);
	my $name = charnames::viacode($cp);
	print B "$r\t$cp\t$h\t$name\tCount:$hash{$r}\n";
}


close B;
