#doubles_merge_v1_5_8.pl
#command line:
#script.pl -d -dic input.txt doubles.txt output.txt
#script.pl -d -list input.txt doubles.txt output.txt
#script.pl -m -[alr] input.txt doubles.txt output.txt
#script.pl -m -[alr] -cb[12] input.txt doubles.txt output.txt
#script.pl -pile input.txt output.txt

use strict;
use warnings;

use Time::HiRes;
my $start = Time::HiRes::gettimeofday();

use open ':utf8';
use POSIX qw(ceil floor);

local $| = 1;

my $ErrorCount = 0;

#___________________________________Rome
#Roman.pm version 1.21
#Roman - Perl module for conversion between Roman and Arabic numerals.
#http://www.cpan.org/


#Domain of valid Roman numerals is limited to less than 4000, since
#proper Roman digits for the rest are not available in ASCII.

our %roman2arabic = qw(I 1 V 5 X 10 L 50 C 100 D 500 M 1000);
my %roman_digit = qw(1 IV 10 XL 100 CD 1000 MMMMMM);
my @figure = reverse sort keys %roman_digit;
#my %roman_digit;
$roman_digit{$_} = [split(//, $roman_digit{$_}, 2)] foreach @figure;

sub isroman($) {
	my $arg = shift;
	$arg ne '' and
		$arg =~ /^(?: M{0,3})
		(?: D?C{0,3} | C[DM])
		(?: L?X{0,3} | X[LC])
		(?: V?I{0,3} | I[VX])$/ix;
}

sub arabic($) {
	my $arg = shift;
	isroman $arg or return undef;
	my($last_digit) = 1000;
	my($arabic);
	foreach (split(//, uc $arg)) {
		my($digit) = $roman2arabic{$_};
	$arabic -= 2 * $last_digit if $last_digit < $digit;
	$arabic += ($last_digit = $digit);
}
$arabic;
}

sub Roman($) {
	my $arg = shift;
	0 < $arg and $arg < 4000 or return undef;
	my($x, $roman);
	foreach (@figure) {
		my($digit, $i, $v) = (int($arg / $_), @{$roman_digit{$_}});
		if (1 <= $digit and $digit <= 3) {
			$roman .= $i x $digit;
		} elsif ($digit == 4) {
			$roman .= "$i$v";
		} elsif ($digit == 5) {
			$roman .= $v;
		} elsif (6 <= $digit and $digit <= 8) {
			$roman .= $v . $i x ($digit - 5);
		} elsif ($digit == 9) {
			$roman .= "$i$x";
		}
		$arg -= $digit * $_;
		$x = $i;
	}
	$roman;
}

sub roman($) {
	lc Roman shift;
}

#___________________________________Rome

sub trim
{
	my $s = shift;
	$s =~ s/^\s+//;
	$s =~ s/\s+$//;
	return $s;
}

sub Execution_time{ 

	my $end = Time::HiRes::gettimeofday();
	my $stat_perf = $end - $start;
	my $min = $stat_perf / 60;
	my $sec = ($stat_perf * 1000 % 60000) / 1000;
	my $elapsed = sprintf("%02u:%06.3f", $min, $sec);

	return "\n\nExecution time: " . $elapsed . " (min:sec.msec)\n";

}



sub progressinfile {
	my $state = shift;
	my $max = shift;
	my $message = shift;

	my $percent = floor(($state/$max)*100);

	if (!defined $p::percent)
	{
	$p::percent = 0;
	}

	if ($percent > $p::percent)
	{

		for(my $k=0;$k<$percent;$k=$k+5)
		{
			print "|";
		}
	
		print "$percent% $message";
		print "\r";

		$p::percent = $percent;

	}

}




my @articles = ();
my %hash = ();
my @dub = ();



if ((@ARGV == 5) && ($ARGV[0] eq '-d') && ($ARGV[1] eq '-dic'))
{
	open(E, ">error.log") || die "could not open file: $!";
	&removedoublesonly;
	print "\rDone  ";
	print &Execution_time;
	if ($ErrorCount > 0)
	{
		print "Number of Errors: " . $ErrorCount . " (Check error.log)\n";
	}
	close E;

}
elsif ((@ARGV == 5) && ($ARGV[0] eq '-d') && ($ARGV[1] eq '-list'))
{
	&removeinlist;
	print &Execution_time;
}
elsif ((@ARGV == 5) && ($ARGV[0] eq '-m') && ($ARGV[1] =~ /^-[alr]$/))
{
	open(E, ">error.log") || die "could not open file: $!";
	&removedoublesandmerge;
	print "\rDone  ";
	print &Execution_time;
	if ($ErrorCount > 0)
	{
		print "Number of Errors: " . $ErrorCount . " (Check error.log)\n";
	}
	close E;
}
elsif ((@ARGV == 6) && ($ARGV[0] eq '-m') && ($ARGV[1] =~ /^-[alr]$/) && ($ARGV[2] =~ /^-cb[12]$/))
{
	open(E, ">error.log") || die "could not open file: $!";
	&removedoublesandmerge;
	print "\rDone  ";
	print &Execution_time;
	if ($ErrorCount > 0)
	{
		print "Number of Errors: " . $ErrorCount . " (Check error.log)\n";
	}
	close E;
}
elsif ((@ARGV == 3) && ($ARGV[0] eq '-pile'))
{
	open(E, ">error.log") || die "could not open file: $!";
	&pileheadwords;
	print "\rDone  ";
	print &Execution_time;
	if ($ErrorCount > 0)
	{
		print "Number of Errors: " . $ErrorCount . " (Check error.log)\n";
	}
	close E;
}
else
{
	print "  ________________________________\n";
	print "  Invalid command line.\n";
	print "  ________________________________\n";
}



sub removedoublesonly{
	&fillarray;
	&remove;
	&write;
}

sub removedoublesandmerge{
	&fillarray;
	&remove;
	&merge;
	&write;
}


sub pileheadwords{
	&fillarray;
	&pileandwrite;
}


sub meth1{
	my $s = shift;
	my $bak = $s;


	while ($s =~ /^(|.*?[^\\])\{(|.*?[^\\])\}(.*)$/)
	{
		$s = "$1$3";
	}

	$s =~ s/\x20\x20/\x20/g;
	$s =~ s/^\s+//;
	$s =~ s/\s+$//;

	if (length($s) == 0)
	{
		$s = $bak;
	}

	return $s;
}




sub printarray {
	my ($ref, $line_num) = @_;
	my @s = @{$ref};
	my @hw = ();
	foreach (@s)
	{
		if ($_ =~ /^[^\t\x20].*$/)
		{
			s/\s+$//;
			push @hw, $_;
		}
	}


	if (@hw > 100)
	{

		$ErrorCount++;
		my $b_line = $line_num - (@hw - 1);
		print E "Too many headwords on line $b_line. Input file does not appear to be a dictionary.\n";

	}

	my $art = "";
	my $checkart = '';


	for(my $i = 0; $i < @s; $i++)
	{
		if ($s[$i] =~ /^[\t\x20].*$/)
		{
			$art = "$art$s[$i]\n";
			$checkart .= trim($s[$i]);
			
			if ((trim($s[$i]) ne '') && ($s[$i - 1] eq ''))
			{

				my $b_line = $line_num + $i - (@hw - 1);				
				$ErrorCount++;
				print E "Format error on line $b_line\n";
				
			}

			
		}
	}

	if ($checkart eq '')
	{

		$ErrorCount++;
		my $b_line = $line_num + 1;
		print E "Format error on line $b_line\n";
		
	}


	if ((@ARGV == 3) && ($ARGV[0] eq '-pile')){

		my $ind = @articles;
		my $allhw = join("\n", @hw);
		my @tmp = ($allhw, $art, $ind);
		push  @articles, [@tmp];

	}
	else{

		foreach(@hw)
		{
			my $ind = @articles;


			my $ts = $_;
			my $bs = $_;

			if ((@ARGV == 6) && ($ARGV[2] eq '-cb1'))
			{
				$ts = meth1($ts);
				my $ck = $hash{$ts};
				if (defined $ck)
				{
				
					if (length($bs) < length($ck))
					{
						$hash{$ts} = $bs;
					}
				}
				else
				{
					$hash{$ts} = $bs;
				}

			}


			if ((@ARGV == 6) && ($ARGV[2] eq '-cb2'))
			{

				$ts = meth1($ts);
				my $ck = $hash{$ts};
				if (defined $ck)
				{
				
					if (length($bs) > length($ck))
					{
						$hash{$ts} = $bs;
					}
				}
				else
				{
					$hash{$ts} = $bs;
				}

			}



			my @tmp = ($ind, $ts, $bs, $art);
			push  @articles, [@tmp] 
		}

	}

}





sub fillarray {

	if ((@ARGV == 3) && ($ARGV[0] eq '-pile'))
	{ 
		open(A, "$ARGV[1]") || die "could not open file: $!";
		open(C, ">$ARGV[2]") || die "could not open file: $!";
	}
	elsif ((@ARGV == 6) && ($ARGV[2] =~ /^-cb[12]$/))
	{
		open(A, "$ARGV[3]") || die "could not open file: $!";
		open(C, ">$ARGV[5]") || die "could not open file: $!";
	}
	else
	{
		open(A, "$ARGV[2]") || die "could not open file: $!";
		open(C, ">$ARGV[4]") || die "could not open file: $!";
	}


	my $count = 0;
	my $hw_start = 0;
	my $filesize = -s A;


	my @arr1 = ();
	my $flag = 0;



	while (<A>) {
		s/[\x0A\x0D]//g;


		$count++;
		if (($count % 2) == 0)
		{
			my $offset = tell(A);
			progressinfile($offset, $filesize, "");
		}


		if ($_ =~ /^(\x{feff})(.*)$/)
		{
			print C "$1";
			$_ = $2;
		}


		if ($_ =~ /^\#(NAME.*|INDEX_LANGUAGE.*|CONTENTS_LANGUAGE.*)$/)
		{
			print C "\#$1\n";
		}
		elsif (($_ =~ /^$/) && (@arr1 == 0))
		{
			print C "$_\n";
		}
		elsif (($_ =~ /^[\t\x20].*$/) && (@arr1 == 0))
		{
			print C "$_\n";
		}
		elsif ($_ =~ /^[^\t\x20].*$/)
		{

			if (($flag == 0) || ($flag == 2))
			{
				if (@arr1 > 0)
				{
					printarray(\@arr1, $hw_start);
				}
				@arr1 = ();
				push @arr1, $_;
			}
			else
			{
				push @arr1, $_;
			} 

			$flag = 1;
			$hw_start = $count;
		}
		else
		{
			push @arr1, $_;
			$flag = 2;

		}

	}


	if (@arr1 > 0)
	{
		printarray(\@arr1, $hw_start);
		@arr1 = ();
	}

	my $offset = tell(A);
	if ($offset > 0)
	{
		progressinfile($offset, $filesize, "");
	}

	print "\n\nWait!";

	close A;
	close C;
}

sub write {

	if ((@ARGV == 6) && ($ARGV[2] =~ /^-cb[12]$/))
	{
		open(B, ">$ARGV[4]") || die "could not open file: $!";
		open(C, ">>$ARGV[5]") || die "could not open file: $!";
	}
	else
	{
		open(B, ">$ARGV[3]") || die "could not open file: $!";
		open(C, ">>$ARGV[4]") || die "could not open file: $!";
	}

	my @sorted = sort {$a->[0] <=> $b->[0]} @articles;
	@articles = ();

	foreach(@sorted)
	{   


		my $row = $_;
		my $ck = $hash{$row->[1]};
		$ck = '' unless defined $ck;

		if ((@ARGV == 6) && ($ARGV[2] =~ /^-cb[12]$/) && ($ck ne ''))
		{
			print C "$ck\n";
		}
		else
		{
			
			print C "$row->[2]\n";
		}

		if (@{$row} > 4)
		{

			for my $j ( 3 .. $#{$row} )
			{


				if ($ARGV[1] eq '-a')
				{ 

					my $i = $j - 2;
					print C "\t\{\{Arabic\}\}$i\{\{\/Arabic\}\}\n";
					print C "$row->[$j]";

				}
				elsif ($ARGV[1] eq '-l')
				{ 

					if ($j == 3)
					{
						print C "$row->[$j]";
					}
					else
					{
						print C "\t\x{2014}\x{2014}\x{2014}\x{2014}\x{2014}\x{2014}\x{2014}\x{2014}\n";
						print C "$row->[$j]";
					}
				}
				else
				{ 

					my $i = $j - 2;
					my $romnum = Roman($i);
					print C "\t\{\{Roman\}\}$romnum\{\{\/Roman\}\}\n";
					print C "$row->[$j]";

				}


			}


		}
		else
		{
			print C $row->[3];
		}



	}

	foreach(@dub)
	{
		print B "$_->[2]\n$_->[3]";
	}

	close B;
	close C;
}



sub remove {

	my @tmp = ();


	my @sorted = sort { ($a->[1] cmp $b->[1]) || ($a->[3] cmp $b->[3])} @articles;
	@articles = ();


	while (my $element = shift(@sorted)) {

		if (@tmp == 0)
		{
			push @tmp, $element;
		}
		else
		{
			if (($element->[1] eq $tmp[$#tmp][1]) && ($element->[3] eq $tmp[$#tmp][3]))
			{
				push @dub, $element;
			}
			else
			{
				push @tmp, $element;
			}

		}

	}


	@articles = @tmp;
	@tmp = ();
}



sub merge {

	my @tmp = ();
	my $op = shift;

	my @sorted = sort { ($a->[1] cmp $b->[1]) || ($a->[0] <=> $b->[0])} @articles;
	@articles = ();

	while (my $element = shift(@sorted))
	{

		if (@tmp == 0)
		{
			push @tmp, $element;
		}
		else
		{

			if ($element->[1] eq $tmp[$#tmp][1])
			{
				push @{ $tmp[$#tmp] }, $element->[3];
			}
			else
			{
				push @tmp, $element;
			}

		}

	}

	@articles = @tmp;
	@tmp = ();
}


sub pileandwrite {


	open(C, ">>$ARGV[2]") || die "could not open file: $!";


	my @sorted = sort {$a->[1] cmp $b->[1]} @articles;
	@articles = ();


	while (my $element = shift(@sorted))
	{

		if (@articles == 0)
		{
			push @articles, $element;
		}
		else
		{

			if ($element->[1] eq $articles[$#articles][1])
			{

				$articles[$#articles][0] .= "\n$element->[0]";

			}
			else
			{
				push @articles, $element;
			}

		}

	}

	@sorted = ();

	@sorted = sort {$a->[2] <=> $b->[2]} @articles;
	@articles = ();

	foreach(@sorted)
	{   

		my $row = $_;
		print C "$row->[0]\n$row->[1]";

	}

	close C;
}


sub removeinlist {
	open(A, "$ARGV[2]") || die "could not open file: $!";
	open(B, ">$ARGV[3]") || die "could not open file: $!";
	open(C, ">$ARGV[4]") || die "could not open file: $!";

	my @strings = ();
	my $linecounter = 0;



	while (<A>) {
		s/[\x0A\x0D]//g;

		if ($_ =~ /^(\x{feff})(.*)$/)
		{
			print C "$1";
			$_ = $2;
		}


		if (($_ =~ /^$/) ne 1)
		{
			my $bs = $_;
			my $ts = $_;
			$ts =~ s/\s+$//;
			if ($ts eq '')
			{
				$ts = $bs;
			}

			$linecounter++;
			my @tmp = ($ts, $linecounter);
			push  @strings, [@tmp];

		}


	}

	my @sorted = sort { ($a->[0] cmp $b->[0])} @strings;
	@strings = ();

	my @new = ();


	while (my $element = shift(@sorted)){

		if (@new == 0)
		{
			push @new, $element;
		}
		else
		{
			if ($element->[0] eq $new[$#new][0])
			{
				print B "$element->[0]\n";
			}
			else
			{
				push @new, $element;
			}

		}

	}

	my @result = sort { ($a->[1] <=> $b->[1])} @new;
	@new = ();

	foreach(@result)
	{
		print C "$_->[0]\n";
	}

	close A;
	close B;
	close C;
}

