#sort_dsl_art_v1_0_5.pl
#command line:
#script.pl -b input.txt output.txt
#script.pl -o input.txt output.txt

#script.pl -b -i1 input.txt output.txt
#script.pl -b -i2 input.txt output.txt
#script.pl -b -i3 input.txt output.txt

#script.pl -o -i1 input.txt output.txt
#script.pl -o -i2 input.txt output.txt
#script.pl -o -i3 input.txt output.txt

#script.pl -list input.txt output.txt
#script.pl -list -i input.txt output.txt

#input.txt - UTF-8

use strict;
use warnings;

use Time::HiRes;
my $start = Time::HiRes::gettimeofday();

use POSIX qw(ceil floor);
use open ':utf8';

local $| = 1;

my $ErrorCount = 0;


if ((@ARGV == 3) && ($ARGV[0] =~ /^-b|-o$/))
	{
		open(E, ">error.log") || die "could not open file: $!";
		&main;
		print "\n\nDone  ";
		print &Execution_time;
		if ($ErrorCount > 0)
		{
			print "Number of Errors: " . $ErrorCount . " (Check error.log)\n";
		}
		close E;

	}
elsif ((@ARGV == 4) && ($ARGV[0] =~ /^-b|-o$/) && ($ARGV[1] =~ /^-i1|-i2|-i3$/))
	{
		open(E, ">error.log") || die "could not open file: $!";
		&main;
		print "\n\nDone  ";
		print &Execution_time;
		if ($ErrorCount > 0)
		{
			print "Number of Errors: " . $ErrorCount . " (Check error.log)\n";
		}
		close E;

	}
elsif ((@ARGV == 3) && ($ARGV[0] =~ /^-list$/))
	{
		&sortinlist;
		print &Execution_time;
	}
elsif ((@ARGV == 4) && ($ARGV[0] eq "-list") && ($ARGV[1] eq "-i"))
	{
		&sortinlist;
		print &Execution_time;
	}
	else
	{
		print "  ________________________________\n";
		print "  Invalid command line.\n";
		print "  ________________________________\n";
		exit;
	}


sub main {
			

if ((@ARGV == 3) && ($ARGV[0] =~ /^-b|-o$/))
	{
		open(A, "$ARGV[1]") || die "could not open file: $!";
		open(B, ">$ARGV[2]") || die "could not open file: $!";
	}
	else
	{
		open(A, "$ARGV[2]") || die "could not open file: $!";
		open(B, ">$ARGV[3]") || die "could not open file: $!";
	}

my $count = 0;
my $filesize = -s A;
my $hw_start = 0;

our @arr1 = ();
my $flag = 0;
our @arr2 = ();


sub trim
{
	my $s = shift;
	$s =~ s/^\s+//;
	$s =~ s/\s+$//;
	return $s;
}

sub Execution_time{ 

	my $end = Time::HiRes::gettimeofday();
	my $stat_perf = $end - $start;
	my $min = $stat_perf / 60;
	my $sec = ($stat_perf * 1000 % 60000) / 1000;
	my $elapsed = sprintf("%02u:%06.3f", $min, $sec);

	return "\n\nExecution time: " . $elapsed . " (min:sec.msec)\n";

}



sub progressinfile {
	my $state = shift;
	my $max = shift;
	my $message = shift;

	my $percent = floor(($state/$max)*100);

	if (!defined $p::percent)
	{
	$p::percent = 0;
	}

	if ($percent > $p::percent)
	{

		for(my $k=0;$k<$percent;$k=$k+5)
		{
			print "|";
		}
	
		print "$percent% $message";
		print "\r";

		$p::percent = $percent;

	}

}



sub printarray {

	my ($ref, $line_num) = @_;
	my @s = @{$ref};
	my @hw = ();

	foreach (@s)
	{
		if ($_ =~ /^[^\t\x20].*$/)
		{
			s/\s+$//;
			push @hw, $_;
		}
	}


	if (@hw > 100)
	{

		$ErrorCount++;
		my $b_line = $line_num - (@hw - 1);
		print E "Too many headwords on line $b_line. Input file does not appear to be a dictionary.\n";

	}



	my $art = "";
	my $checkart = '';


	for(my $i = 0; $i < @s; $i++)
	{
		if ($s[$i] =~ /^[\t\x20].*$/)
		{
			$art = "$art$s[$i]\n";
			$checkart .= trim($s[$i]);
			
			if ((trim($s[$i]) ne '') && ($s[$i - 1] eq ''))
			{

				my $b_line = $line_num + $i - (@hw - 1);				
				$ErrorCount++;
				print E "Format error on line $b_line\n";
				
			}

			
		}
	}


	if ($checkart eq '')
	{

		$ErrorCount++;
		my $b_line = $line_num + 1;
		print E "Format error on line $b_line\n";
		
	}


	foreach(@hw)
	{
		my $ind = @arr2;
		my @tmp = ($ind, $_, $art);
		push  @arr2, [@tmp]
	}

}




while (<A>) {
s/[\x0A\x0D]//g;

$count++;

if (($count % 2) == 0)
	{
		my $offset = tell(A);
		progressinfile($offset, $filesize, "");
	}



if ($_ =~ /^(\x{feff})(.*)$/)
	{
		print B "$1";
		$_ = $2;
	}


if ($_ =~ /^\#(NAME.*|INDEX_LANGUAGE.*|CONTENTS_LANGUAGE.*)$/)
	{
		print B "\#$1\n";
	}
elsif (($_ =~ /^$/) && (@arr1 == 0))
	{
		print B "$_\n";
	}
elsif (($_ =~ /^[\t\x20].*$/) && (@arr1 == 0))
	{
		print B "$_\n";
	}
elsif ($_ =~ /^[^\t\x20].*$/)
	{

	if (($flag == 0) || ($flag == 2))
		{

		if (@arr1 > 0)
			{
				printarray(\@arr1, $hw_start);
			}

			@arr1 = ();
			push @arr1, $_;
		}
		else
		{
			push @arr1, $_;
		} 

	$flag = 1;
	$hw_start = $count;

	}
	else
	{

		push @arr1, $_;
		$flag = 2;

	}

}


if (@arr1 > 0)
{
	printarray(\@arr1, $hw_start);
	@arr1 = ();
}

my $offset = tell(A);
progressinfile($offset, $filesize, "");


&meth2;


sub meth2 {

my @sorted = ();

if ((@ARGV == 3) && ($ARGV[0] eq '-b'))
		{
			@sorted = sort {($a->[1] cmp $b->[1]) || ($a->[2] cmp $b->[2])} @arr2;
		}
elsif ((@ARGV == 3) && ($ARGV[0] eq '-o'))
		{
			@sorted = sort {($a->[1] cmp $b->[1]) || ($a->[0] <=> $b->[0])} @arr2;
		}
elsif ((@ARGV == 4) && ($ARGV[0] eq '-b') && ($ARGV[1] eq '-i1'))
		{
			@sorted = sort {("\L$a->[1]" cmp "\L$b->[1]") || ($a->[2] cmp $b->[2])} @arr2;
		}
elsif ((@ARGV == 4) && ($ARGV[0] eq '-b') && ($ARGV[1] eq '-i2'))
		{
			@sorted = sort {(clearhw($a->[1]) cmp clearhw($b->[1])) || ($a->[2] cmp $b->[2])} @arr2;
		}
elsif ((@ARGV == 4) && ($ARGV[0] eq '-b') && ($ARGV[1] eq '-i3'))
		{
			@sorted = sort {(clearhw("\L$a->[1]") cmp clearhw("\L$b->[1]")) || ($a->[2] cmp $b->[2])} @arr2;
		}
elsif ((@ARGV == 4) && ($ARGV[0] eq '-o') && ($ARGV[1] eq '-i1'))
		{
			@sorted = sort {("\L$a->[1]" cmp "\L$b->[1]") || ($a->[0] <=> $b->[0])} @arr2;
		}
elsif ((@ARGV == 4) && ($ARGV[0] eq '-o') && ($ARGV[1] eq '-i2'))
		{
			@sorted = sort {(clearhw($a->[1]) cmp clearhw($b->[1])) || ($a->[0] <=> $b->[0])} @arr2;
		}
elsif ((@ARGV == 4) && ($ARGV[0] eq '-o') && ($ARGV[1] eq '-i3'))
		{
			@sorted = sort {(clearhw("\L$a->[1]") cmp clearhw("\L$b->[1]")) || ($a->[0] <=> $b->[0])} @arr2;
		}
		else
		{

		}

@arr2 = ();


foreach(@sorted)
		{
			my $row = $_;
			print B "$row->[1]\n$row->[2]";
		}

}



sub clearhw {

my $s = shift;
my $bak = $s;

while ($s =~ /^(|.*?[^\\])\{(|.*?[^\\])\}(.*)$/)
{
	$s = "$1$3";
}

$s =~ s/\\([\@\#\^\~\[\]\{\}\(\)])/$1/g;


if (length($s) == 0)
	{
		$s = $bak;
	}

return $s;

}



close A;
close B;

}
 



sub sortinlist {

if ((@ARGV == 3) && ($ARGV[0] =~ /^-list$/))
{
	open(A, "$ARGV[1]") || die "could not open file: $!";
	open(B, ">$ARGV[2]") || die "could not open file: $!";
}
else
{
	open(A, "$ARGV[2]") || die "could not open file: $!";
	open(B, ">$ARGV[3]") || die "could not open file: $!";
}

my @strings = ();
my $linecounter = 0;



while (<A>) {
s/[\x0A\x0D]//g;

if ($_ =~ /^(\x{feff})(.*)$/)
	{
		print B "$1";
		$_ = $2;
	}


if (($_ =~ /^$/) ne 1)
	{
		push  @strings, $_;
	}


}

my @sorted = ();



if ($ARGV[1] eq "-i")
{
	@sorted = sort { "\L$a" cmp "\L$b" } @strings;
}
else
{
	@sorted = sort { $a cmp $b } @strings;	
}


@strings = ();


foreach(@sorted)
{
	print B "$_\n";
}

close A;
close B;

}
