#tagscorrector_v1_0_7_beta.pl
#command line:
#script.pl -1 input.txt output.txt
#script.pl -2 input.txt output.txt


use strict;
use warnings;

use Time::HiRes;
my $start = Time::HiRes::gettimeofday();

use POSIX qw(ceil floor);
use open ':utf8';

local $| = 1;

my $ErrorCount = 0;


if ((@ARGV == 3) && ($ARGV[0] =~ /^-[123]$/))
{
	open(E, ">error.log") || die "could not open file: $!";
	&processasdic;
	print &Execution_time;
	if ($ErrorCount > 0)
	{
		print "Number of Errors: " . $ErrorCount . " (Check error.log)\n";
	}
	close E;


}
else
{
	print "  ________________________________\n";
	print "  Invalid command line.\n";
	print "  ________________________________\n";
	exit;
}


sub trim
{
	my $s = shift;
	$s =~ s/^\s+//;
	$s =~ s/\s+$//;
	return $s;
}


sub Execution_time{ 

	my $end = Time::HiRes::gettimeofday();
	my $stat_perf = $end - $start;
	my $min = $stat_perf / 60;
	my $sec = ($stat_perf * 1000 % 60000) / 1000;
	my $elapsed = sprintf("%02u:%06.3f", $min, $sec);

	return "\n\nExecution time: " . $elapsed . " (min:sec.msec)\n";

}



sub progressinfile {
	my $state = shift;
	my $max = shift;
	my $message = shift;

	my $percent = floor(($state/$max)*100);

	if (!defined $p::percent)
	{
		$p::percent = 0;
	}

	if ($percent > $p::percent)
	{

		for(my $k=0;$k<$percent;$k=$k+5)
		{
			print "|";
		}

		print "$percent% $message";
		print "\r";

		$p::percent = $percent;

	}

}






sub tags1 {
	my $s = shift;
	my $tname = shift;
	$s =~ s/,/\&\#44;/g;

	$s =~ s/\[$tname\]/\[$tname,\]/gi;
	$s =~ s/\[\/$tname\]/\[,\/$tname\]/gi;


	while ($s =~ /^(.*?)\[($tname),\]([^,]*?)\[,(\/$tname)\](.*)$/si)
	{
		$s = "$1\[$2\]$3\[$4\]$5";
	}


	$s =~ s/\[($tname),\]/\{\{$1\}\}/gi;
	$s =~ s/\[,(\/$tname)\]/\{\{$1\}\}/gi;


	$s =~ s/\[$tname\]/\[$tname,\]/gi;
	$s =~ s/\[\/$tname\]/\[,\/$tname\]/gi;


	while ($s =~ /^(.*?)(\[$tname,\])([^,]*?)\[($tname),\]([^,]*?)\[,(\/$tname)\](.*)$/si)
	{
		$s = "$1$2$3\{\{$4\}\}$5\{\{$6\}\}$7";
	}

	$s =~ s/\[($tname),\]/\[$tname\]/gi;
	$s =~ s/\[,(\/$tname)\]/\[\/$tname\]/gi;


	$s =~ s/\&\#44;/,/g;
	return $s;
}


sub color1 {
	my $s = shift;
	my @info = ();
	$s =~ s/,/\&\#44;/g;
	$s =~ s/(^|[^\\])\[c\]/$1\[c color\]/g;

	$s =~ s/(\[c [a-z]+)\]/$1,\]/g;
	$s =~ s/\[(\/c\])/\[,$1/g;
	while ($s =~ /^(.*?)\[(c [a-z]+),\]([^,]*?)\[,(\/c)\](.*)$/s)
	{
		$s = "$1\[$2\]$3\[$4\]$5";
	}

	$s =~ s/\[(c [a-z]+),\]/\{\{$1\}\}/g;
	$s =~ s/\[,(\/c)\]/\{\{$1\}\}/g;

	$s =~ s/(\[c [a-z]+\])/,$1,/g;
	$s =~ s/(\[\/c\])/,$1,/g;

	my @temparr = split(/,/, $s);

	for (my $i = 0; $i < @temparr; $i++) 
	{ 

		if ($temparr[$i] =~ /^\[c [a-z]+\]$/i)
		{

			my $tmp = $temparr[$i];

			if (@info > 0)
			{
				$temparr[$i] = "\[\/c\]$temparr[$i]";
			}

			push @info, $tmp;

		}
		elsif ($temparr[$i] =~ /^\[\/c\]$/i)
		{

			if (@info > 0)
			{
				delete $info[$#info];
			}

			if (@info > 0)
			{
				$temparr[$i] = "\[\/c\]$info[$#info]";
			}


		}
		else
		{


		}

	}



	$s = join("", @temparr);
	@temparr = ();
	$s =~ s/\&\#44;/,/g;
	$s =~ s/\[c color\]/\[c\]/g;


	return $s;
}







sub meth1 {
	my $s = shift;

	if ($ARGV[0] eq '-1')
	{
		$s = tags1($s, 'b');
		$s = tags1($s, 'i');
		$s = tags1($s, 'u');
		$s = tags1($s, 't');
		$s = tags1($s, 'sub');
		$s = tags1($s, 'sup');
	}
	else
	{
		$s = color1($s);
	}

	return $s;
}



sub processasdic {


	open(A, "$ARGV[1]") || die "could not open file: $!";
	open(B, ">$ARGV[2]") || die "could not open file: $!";


	my @arr1 = ();
	my $flag = 0;

	sub printarray {

	my ($ref, $line_num) = @_;
	my @s = @{$ref};

		my @hw = ();
		foreach (@s)
		{
			if ($_ =~ /^[^\t\x20].*$/)
			{
				s/\s+$//;
				push @hw, $_;
			}
		}


		if (@hw > 100)
		{
	
			$ErrorCount++;
			my $b_line = $line_num - (@hw - 1);
			print E "Too many headwords on line $b_line. Input file does not appear to be a dictionary.\n";
	
		}



		my $art = "";
		my $checkart = '';

		for(my $i = 0; $i < @s; $i++)
		{
			if ($s[$i] =~ /^[\t\x20].*$/)
			{

				$art = "$art$s[$i]\n";
				$checkart .= trim($s[$i]);

				if ((trim($s[$i]) ne '') && ($s[$i - 1] eq ''))
				{
	
					my $b_line = $line_num + $i - (@hw - 1);				
					$ErrorCount++;
					print E "Format error on line $b_line\n";
					
				}


			}
		}


		if ($checkart eq '')
		{
	
			$ErrorCount++;
			my $b_line = $line_num + 1;
			print E "Format error on line $b_line\n";
			
		}

		$art = meth1($art);

		foreach(@hw)
		{
			print B "$_\n";
			print B $art;
		}

	}


	my $hw_start = 0;
	my $count = 0;
	my $filesize = -s A;


	while (<A>) {
	s/[\x0A\x0D]//g;



		$count++;
		if (($count % 2) == 0)
		{
			my $offset = tell(A);
			progressinfile($offset, $filesize, "");
		}


		if ($_ =~ /^(\x{feff})(.*)$/)
		{
			print B "$1";
			$_ = $2;
		}


		if ($_ =~ /^\#(NAME.*|INDEX_LANGUAGE.*|CONTENTS_LANGUAGE.*)$/)
		{
			print B "\#$1\n";
		}
		elsif (($_ =~ /^$/) && (@arr1 == 0))
		{
			print B "$_\n";
		}
		elsif (($_ =~ /^[\t\x20].*$/) && (@arr1 == 0))
		{
			print B "$_\n";
		}
		elsif ($_ =~ /^[^\t\x20].*$/)
		{

			if (($flag == 0) || ($flag == 2))
			{
				if (@arr1 > 0)
				{
					printarray(\@arr1, $hw_start);
				}
				@arr1 = ();
				push @arr1, $_;
			}
			else
			{
				push @arr1, $_;
			} 

			$flag = 1;
			$hw_start = $count;
		}
		else
		{
			push @arr1, $_;
			$flag = 2;

		}

	}


	if (@arr1 > 0)
	{
		printarray(\@arr1, $hw_start);
		@arr1 = ();
	}

	my $offset = tell(A);
	progressinfile($offset, $filesize, "");


	close A;
	close B;

}

